package stream.dashboard;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import stream.Configurable;
import stream.Data;
import stream.ProcessContext;
import stream.Processor;
import stream.StatefulProcessor;
import stream.io.SourceURL;
import stream.net.JSONReceiver;
import stream.runtime.ProcessContextImpl;
import stream.runtime.setup.ObjectFactory;
import stream.util.XMLUtils;

public class Dashboard extends JFrame implements StatefulProcessor,
		Configurable {

	static Logger log = LoggerFactory.getLogger(Dashboard.class);

	/** The unique class ID */
	private static final long serialVersionUID = 8494360239506665376L;

	final List<Widget> widgets = new ArrayList<Widget>();
	final List<JInternalFrame> frames = new ArrayList<JInternalFrame>();
	final JDesktopPane content = new JDesktopPane();
	final JPanel display = createContentPane();
	final List<Widget> processors = new ArrayList<Widget>();
	final ProcessContext ctx = new ProcessContextImpl();
	String message = null;
	JSONReceiver receiver;
	SourceURL config;
	protected final List<String> nameSpaces = new ArrayList<String>();

	public Dashboard() {
		nameSpaces.add("stream.dashboard");
		setTitle("Dashboard");
		setSize(1024, 768);
		content.setBackground(Color.WHITE);
		display.setBackground(Color.WHITE);
		setContentPane(display);

		WidgetMover m = new WidgetMover(this);
		addMouseMotionListener(m);
		display.addMouseMotionListener(m);
		display.addMouseListener(m);
	}

	public JPanel createContentPane() {
		return new JPanel(null);
	}

	public void init(URL url) throws Exception {
		DocumentBuilder db = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();
		Document doc = db.parse(url.openStream());
		init(doc);
	}

	public void init(SourceURL url) throws Exception {
		DocumentBuilder db = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();
		Document doc = db.parse(url.openStream());
		init(doc);
	}

	public void init(Document doc) throws Exception {
		Element root = doc.getDocumentElement();
		init(root);
	}

	public void init(Element root) throws Exception {

		ObjectFactory factory = ObjectFactory.newInstance();
		for (String pkg : nameSpaces) {
			factory.addPackage(pkg.trim());
		}
		String host = "0.0.0.0";
		try {
			if (root.hasAttribute("address")) {
				host = root.getAttribute("address");
			}

			Integer port = new Integer(root.getAttribute("port"));
			log.info("Listening for JSON streams on {}:{}", host, port);

			this.receiver = new JSONReceiver(host, port);
			this.receiver.registerListener(new DataListener() {
				public void dataArrived(Data item) {
					process(item);
				}
			});
			this.receiver.start();
		} catch (Exception e) {
			log.error("Error: " + e.getMessage());
		}

		try {
			Dimension d = parseDimension(root.getAttribute("ui:size"));
			setSize(d);
		} catch (Exception e) {
		}

		NodeList ch = root.getChildNodes();
		for (int i = 0; i < ch.getLength(); i++) {
			Node c = ch.item(i);
			if (c.getNodeType() == Node.ELEMENT_NODE) {
				Element el = (Element) c;
				log.info("Creating widget from element '{}'", el);
				Widget widget = (Widget) factory.create(el);
				add(widget);

				String loc = el.getAttribute("ui:location");
				if (loc == null || "".equals(loc.trim()))
					loc = "10,10";
				Point pt = parseLocation(loc);
				log.info("Point: {}", pt);
				widget.setLocation(pt);

				String size = el.getAttribute("ui:size");
				if (size == null || "".equals(size.trim()))
					size = "320,240";
				Dimension sz = parseDimension(size);
				log.info("Size: {}", sz);
				widget.setSize(sz);
			}
		}
	}

	public int gridSize() {
		return 10;
	}

	public void add(Widget w) {
		try {
			w.init(ctx);
			if (w instanceof Processor)
				processors.add(w);
			display.add(w);
			widgets.add(w);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public List<Widget> getWidgets() {
		return widgets;
	}

	/**
	 * @see stream.StatefulProcessor#init(stream.ProcessContext)
	 */
	public void init(ProcessContext context) throws Exception {
		try {
			if (config != null) {
				Document doc = XMLUtils.parseDocument(this.config.openStream());
				this.init(doc);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception("Failed to initialize Dashboard: "
					+ e.getMessage());
		}
		this.setVisible(true);
	}

	/**
	 * @see stream.Processor#process(stream.Data)
	 */
	public Data process(Data item) {
		// log.info("Processing item with {} processors", processors.size());
		for (Widget w : processors) {
			// log.info("Processing item with widget {}: {}", w, item);
			if (w.handles(item)) {
				log.debug("Handling item {} with widget {}", item, w);
				w.process(item);
			} else {
				log.debug("Skipping widget {} for item {}", w, item);
			}
		}

		return item;
	}

	/**
	 * @see stream.StatefulProcessor#resetState()
	 */
	public void resetState() throws Exception {
	}

	/**
	 * @see stream.StatefulProcessor#finish()
	 */
	public void finish() throws Exception {
	}

	public void message(String msg) {
		this.message = msg;
		this.repaint();
		this.validate();
	}

	public void paint(Graphics g) {
		super.paint(g);

		if (message != null)
			g.drawString(message, this.getWidth() / 2, 50);
	}

	public Dimension parseDimension(String s) {
		log.info("Parsing dimension from {}", s);
		String[] tok = s.split(",", 2);
		return new Dimension(new Integer(tok[0]), new Integer(tok[1]));
	}

	public Point parseLocation(String s) {
		log.info("Parsing location from {}", s);
		String[] tok = s.split(",", 2);
		return new Point(new Integer(tok[0]), new Integer(tok[1]));
	}

	/**
	 * @return the config
	 */
	public SourceURL getConfig() {
		return config;
	}

	/**
	 * @param config
	 *            the config to set
	 */
	public void setConfig(SourceURL config) {
		this.config = config;
	}

	/**
	 * @see stream.Configurable#configure(org.w3c.dom.Element)
	 */
	public void configure(Element document) {
		if (document != null) {
			try {
				log.info("Building dashboard from nested XML config..");
				this.init(document);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}