package stream.dashboard;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * The default color-mapping.
 * </p>
 * 
 * @author Christian Bockermann &lt;chris@jwall.org&gt;
 * 
 */
public class DefaultColorMapping implements ColorMapping {

	static Logger log = LoggerFactory.getLogger(DefaultColorMapping.class);

	private static DefaultColorMapping colorMapping = new DefaultColorMapping();

	Map<String, Color> colorMappings;

	/**
     * 
     */
	public DefaultColorMapping() {

		colorMappings = new LinkedHashMap<String, Color>();

		try {

			URL url = DefaultColorMapping.class
					.getResource("/colors.properties");
			if (url != null) {
				log.debug("Found color-mapping at: {}", url);

				Map<String, Color> cols = readColorMap(url.openStream());
				addColors(cols);

			} else
				log.warn("No color-mapping found!");

		} catch (Exception e) {
			e.printStackTrace();
		}

		try {

			File colorFile = new File(System.getProperty("user.home")
					+ File.separator + ".colors.properties");
			if (colorFile.exists()) {
				log.debug("Loading custom color mapping from {}", colorFile);
				Map<String, Color> colors = readColorMap(new FileInputStream(
						colorFile));
				addColors(colors);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static ColorMapping getInstance() {
		if (colorMapping == null)
			colorMapping = new DefaultColorMapping();

		return colorMapping;
	}

	public void set(String aspect, String value, Color c) {
		colorMappings.put(aspect + "." + value, c);
	}

	/**
	 * @see org.jwall.web.audit.console.statistics.ColorMapping#map(java.lang.String,
	 *      java.lang.String)
	 */
	public Color map(String aspect, String value) {
		Color color = colorMappings.get(aspect + "." + value);

		if (color == null) {
			log.debug("No color defined for {}={}", aspect, value);
			color = getNextAvailableColor(aspect);
			if (color != null)
				colorMappings.put(aspect + "." + value, color);
		} else
			log.debug("   {}={} ~> " + this.getHtmlCode(color), aspect, value);

		// if no color is defined or found, then just return a plain gray
		//
		if (color == null)
			return Color.lightGray;

		return color;
	}

	public Color getNextAvailableColor(String aspect) {

		int count = 0;
		for (String key : colorMappings.keySet())
			if (key.startsWith(aspect))
				count++;

		log.debug("Aspect '{}' has {} colors defined", aspect, count);

		Color last = colorMappings.get(aspect);
		if (last == null) {
			log.debug("No base-color defined for aspect '{}'!", aspect);
			return null;
		}

		log.debug("  last assigned color for {} is {}", aspect,
				getHtmlCode(last));

		float angle = 60.0f / 360.0f;
		float saturation = 1.0f;
		float brightness = count * (1.0f / 40.0f);

		log.debug(" angle = {}");
		log.debug(" saturation = {}", saturation);
		log.debug(" brightness = {}", brightness);

		last = Color.getHSBColor(angle, saturation, brightness);
		colorMappings.put(aspect + "." + count, last);

		/*
		 * for( int i = 0; i + 1 < count; i++ ){ last = last.brighter(); //last
		 * = last.brighter(); }
		 */

		log.debug("   last.brigther is {}", getHtmlCode(last));

		return last;
	}

	/**
	 * This method maps a html-color code to the corresponding java color.
	 * 
	 * @param code
	 * @return
	 */
	public static Color getColorForCode(String code) {

		log.debug("Checking color from code '{}'", code);

		try {

			if (code.startsWith("#") && code.length() == 7) {

				String c1 = code.substring(1, 3);
				String c2 = code.substring(3, 5);
				String c3 = code.substring(5);

				log.debug("  c1 = {}, c2 = {}, c3 = " + c3, c1, c2);
				int r = Integer.parseInt(c1, 16);
				int g = Integer.parseInt(c2, 16);
				int b = Integer.parseInt(c3, 16);

				log.debug("  r/g/b = " + r + "/" + g + "/" + b);

				Color c = new Color(r, g, b);
				return c;
			}

			log.warn("Invalid color-definition! Only html-color codes are currently supported!");

		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	private void addColors(Map<String, Color> colors) {
		for (String key : colors.keySet())
			colorMappings.put(key, colors.get(key));
	}

	public String getHtmlCode(Color color) {
		String r = Integer.toHexString(color.getRed());
		if (r.length() == 1)
			r = "0" + r;

		String g = Integer.toHexString(color.getGreen());
		if (g.length() == 1)
			g = "0" + g;

		String b = Integer.toHexString(color.getBlue());
		if (b.length() == 1)
			b = "0" + b;

		return "#" + r + g + b;
	}

	public Set<String> getMappedValues(String aspect) {

		SortedSet<String> values = new TreeSet<String>();
		for (String key : this.colorMappings.keySet()) {
			if (key.startsWith(aspect + "."))
				values.add(key.substring(aspect.length() + 1));
		}

		return values;
	}

	public static Map<String, Color> readColorMap(InputStream in) {

		Map<String, Color> colors = new LinkedHashMap<String, Color>();

		try {

			BufferedReader r = new BufferedReader(new InputStreamReader(in));
			String line = r.readLine();
			while (line != null) {

				line = line.trim();
				log.debug("Line: {}", line);
				if (!line.equals("") && line.indexOf("=") > 0
						&& !line.startsWith("#")) {

					int idx = line.indexOf("=");
					if (idx > 0) {

						String key = line.substring(0, idx);
						String code = line.substring(idx + 1);

						if (key.equalsIgnoreCase("AUTO_COLORS")) {

							int i = 0;
							String[] colorCodes = code.split(",");
							for (String cc : colorCodes) {
								Color col = getColorForCode(cc);
								if (col == null) {

								} else {
									colors.put("AUTO_COLORS_" + i, col);
								}
							}
						} else {
							Color color = getColorForCode(code);

							log.debug("Registering color '{}' for key '{}'",
									color, key);
							colors.put(key, color);
						}
					}
				}

				line = r.readLine();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return colors;
	}
}